@@ -1,25 +1,23 @@
-import logging
+import os
 import tarfile
 from pathlib import Path
 from tempfile import NamedTemporaryFile
 import itertools
+from contextlib import ExitStack
 
 from gwdc_python import GWDC
+from gwdc_python.files import FileReference, FileReferenceList
+from gwdc_python.helpers import TimeRange, Cluster
+from gwdc_python.utils import rename_dict_keys
+from gwdc_python.logger import create_logger
 
 from .bilby_job import BilbyJob
 from .event_id import EventID
-from .file_reference import FileReference, FileReferenceList
-from .helpers import TimeRange, Cluster
-from .utils import convert_dict_keys
 from .utils.file_download import _download_files, _save_file_map_fn, _get_file_map_fn
+from .utils.file_upload import check_file
 from .settings import GWCLOUD_ENDPOINT
 
-logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
-
-ch = logging.StreamHandler()
-ch.setLevel(logging.INFO)
-logger.addHandler(ch)
+logger = create_logger(__name__)
 
 
 class GWCloud:
@@ -42,6 +40,48 @@ class GWCloud:
         self.client = GWDC(token=token, endpoint=endpoint)
         self.request = self.client.request  # Setting shorthand for simplicity
 
+    def _upload_supporting_files(self, tokens, file_paths):
+        """
+        Uploads supporting files for a job
+
+        Parameters
+        ----------
+        token : list
+            List of supporting file upload tokens
+        file_path : list
+            List of local file paths to the supporting files to be uploaded
+
+        Returns
+        -------
+        None
+        """
+        query = """
+            mutation SupportingFilesUploadMutation($input: UploadSupportingFilesMutationInput!) {
+                uploadSupportingFiles(input: $input) {
+                    result {
+                        result
+                    }
+                }
+            }
+        """
+        file_paths = map(check_file, file_paths)
+        with ExitStack() as stack:
+            files = [stack.enter_context(file_path.open('rb')) for file_path in file_paths]
+
+            variables = {
+                "input": {
+                    "supportingFiles": [
+                        {"fileToken": token, "supportingFile": f} for token, f in zip(tokens, files)
+                    ]
+                }
+            }
+
+            data = self.request(query=query, variables=variables, authorize=False)
+
+        result = data['upload_supporting_files']['result']['result']
+        if not result:
+            raise Exception("Unable to upload supporting files. An error occurred on the remote side.")
+
     def start_bilby_job_from_string(self, job_name, job_description, private, ini_string, cluster=Cluster.DEFAULT):
         """Submit the parameters required to start a Bilby job, using the contents of an .ini file
 
@@ -55,7 +95,7 @@ class GWCloud:
             True if job should be private, False if public
         ini_string : str
             The contents of a Bilby ini file
-        cluster : .Cluster or str
+        cluster : ~gwdc_python.helpers.Cluster or str
             The name of the cluster to submit the job to
 
         Returns
@@ -68,6 +108,10 @@ class GWCloud:
                 newBilbyJobFromIniString (input: $input) {
                     result {
                         jobId
+                        supportingFiles {
+                            filePath
+                            token
+                        }
                     }
                 }
             }
@@ -90,7 +134,16 @@ class GWCloud:
         }
 
         data = self.request(query=query, variables=variables)
-        job_id = data['newBilbyJobFromIniString']['result']['jobId']
+
+        # Upload any supporting files returned by the job submission
+        tokens, file_paths = [], []
+        for supporting_file in data['new_bilby_job_from_ini_string']['result']['supporting_files']:
+            tokens.append(supporting_file['token'])
+            file_paths.append(supporting_file['file_path'])
+
+        self._upload_supporting_files(tokens, file_paths)
+
+        job_id = data['new_bilby_job_from_ini_string']['result']['job_id']
         return self.get_job_by_id(job_id)
 
     def start_bilby_job_from_file(self, job_name, job_description, private, ini_file, cluster=Cluster.DEFAULT):
@@ -106,7 +159,7 @@ class GWCloud:
             True if job should be private, False if public
         ini_file : str or Path
             Path to an .ini file for running a Bilby job
-        cluster : .Cluster or str
+        cluster : ~gwdc_python.helpers.Cluster or str
             The name of the cluster to submit the job to
 
         Returns
@@ -114,10 +167,18 @@ class GWCloud:
         str
             Message received from server after job submission
         """
+
+        # Change the working directory to the folder containing the ini file, this will make the supporting file upload
+        # search for files relative to the ini file
+        cwd = Path().resolve()
         ini_file = Path(ini_file)
-        with ini_file.open() as f:
-            ini_string = f.read().strip()
-            return self.start_bilby_job_from_string(job_name, job_description, private, ini_string, cluster)
+        try:
+            os.chdir(Path(ini_file).parent)
+            with ini_file.open() as f:
+                ini_string = f.read().strip()
+                return self.start_bilby_job_from_string(job_name, job_description, private, ini_string, cluster)
+        finally:
+            os.chdir(str(cwd))
 
     def get_preferred_job_list(self, search=""):
         """Get list of public Bilby jobs corresponding to a search of "preferred" and a time_range of "Any time"
@@ -136,11 +197,10 @@ class GWCloud:
 
     def _get_job_model_from_query(self, query_data):
         if not query_data:
-            # logger.info('No')
             return None
         return BilbyJob(
             client=self,
-            **convert_dict_keys(
+            **rename_dict_keys(
                 query_data,
                 {'id': 'job_id'}
             )
@@ -154,7 +214,7 @@ class GWCloud:
         ----------
         search : str, optional
             Search terms by which to filter public job list, by default ""
-        time_range : .TimeRange or str, optional
+        time_range : ~gwdc_python.helpers.TimeRange or str, optional
             Time range by which to filter job list, by default TimeRange.ANY
         number : int, optional
             Number of job results to return in one request, by default 100
@@ -197,11 +257,11 @@ class GWCloud:
 
         data = self.request(query=query, variables=variables)
 
-        if not data['publicBilbyJobs']['edges']:
+        if not data['public_bilby_jobs']['edges']:
             logger.info('Job search returned no results.')
             return []
 
-        return [self._get_job_model_from_query(job['node']) for job in data['publicBilbyJobs']['edges']]
+        return [self._get_job_model_from_query(job['node']) for job in data['public_bilby_jobs']['edges']]
 
     def get_job_by_id(self, job_id):
         """Get a Bilby job instance corresponding to a specific job ID
@@ -243,11 +303,11 @@ class GWCloud:
 
         data = self.request(query=query, variables=variables)
 
-        if not data['bilbyJob']:
+        if not data['bilby_job']:
             logger.info('No job matching input ID was returned.')
             return None
 
-        return self._get_job_model_from_query(data['bilbyJob'])
+        return self._get_job_model_from_query(data['bilby_job'])
 
     def get_user_jobs(self, number=100):
         """Obtains a list of Bilby jobs created by the user, filtering based on the search terms
@@ -294,7 +354,7 @@ class GWCloud:
 
         data = self.request(query=query, variables=variables)
 
-        return [self._get_job_model_from_query(job['node']) for job in data['bilbyJobs']['edges']]
+        return [self._get_job_model_from_query(job['node']) for job in data['bilby_jobs']['edges']]
 
     def _get_files_by_job_id(self, job_id):
         query = """
@@ -315,8 +375,8 @@ class GWCloud:
             "jobId": job_id
         }
 
-        data = convert_dict_keys(self.request(query=query, variables=variables))
-        is_uploaded_job = data['bilby_result_files']['is_uploaded_job']
+        data = self.request(query=query, variables=variables)
+        uploaded = data['bilby_result_files']['is_uploaded_job']
 
         file_list = FileReferenceList()
         for file_data in data['bilby_result_files']['files']:
@@ -327,26 +387,27 @@ class GWCloud:
                 FileReference(
                     **file_data,
                     job_id=job_id,
-                    is_uploaded_job=data['bilby_result_files']['is_uploaded_job']
+                    uploaded=uploaded
                 )
             )
 
-        return file_list, is_uploaded_job
+        return file_list, uploaded
 
     def get_files_by_reference(self, file_references):
-        """Obtains file data when provided a FileReferenceList
+        """Obtains file data when provided a :class:`~gwdc_python.files.file_reference.FileReferenceList`
 
         Parameters
         ----------
-        file_references : FileReferenceList
-            Contains the :class:`FileReference` objects for which to download the contents
+        file_references : ~gwdc_python.files.file_reference.FileReferenceList
+            Contains the :class:`~gwdc_python.files.file_reference.FileReference` objects for which
+            to download the contents
 
         Returns
         -------
         list
             List of tuples containing the file path and file contents as a byte string
         """
-        batched = file_references._batch_by_job_id()
+        batched = file_references.batched
 
         file_ids = [
             self._get_download_ids_from_tokens(job_id, job_files.get_tokens())
@@ -354,7 +415,7 @@ class GWCloud:
         ]
 
         file_ids = list(itertools.chain.from_iterable(file_ids))
-        batched_files = FileReferenceList(itertools.chain.from_iterable(batched.values()))
+        batched_files = FileReferenceList(list(itertools.chain.from_iterable(batched.values())))
 
         file_paths = batched_files.get_paths()
         file_uploaded = batched_files.get_uploaded()
@@ -367,18 +428,19 @@ class GWCloud:
         return files
 
     def save_files_by_reference(self, file_references, root_path, preserve_directory_structure=True):
-        """Save files when provided a FileReferenceList and a root path
+        """Save files when provided a :class:`~gwdc_python.files.file_reference.FileReferenceList` and a root path
 
         Parameters
         ----------
-        file_references : FileReferenceList
-            Contains the :class:`FileReference` objects for which to save the associated files
+        file_references : ~gwdc_python.files.file_reference.FileReferenceList
+            Contains the :class:`~gwdc_python.files.file_reference.FileReference` objects for which
+            to save the associated files
         root_path : str or ~pathlib.Path
             Directory into which to save the files
         preserve_directory_structure : bool, optional
             Remove any directory structure for the downloaded files, by default True
         """
-        batched = file_references._batch_by_job_id()
+        batched = file_references.batched
 
         file_ids = [
             self._get_download_ids_from_tokens(job_id, job_files.get_tokens())
@@ -386,7 +448,7 @@ class GWCloud:
         ]
 
         file_ids = list(itertools.chain.from_iterable(file_ids))
-        batched_files = FileReferenceList(itertools.chain.from_iterable(batched.values()))
+        batched_files = FileReferenceList(list(itertools.chain.from_iterable(batched.values())))
 
         file_paths = batched_files.get_output_paths(root_path, preserve_directory_structure)
         file_uploaded = batched_files.get_uploaded()
@@ -447,7 +509,7 @@ class GWCloud:
 
         data = self.request(query=query, variables=variables)
 
-        return data['generateFileDownloadIds']['result']
+        return data['generate_file_download_ids']['result']
 
     def _generate_upload_token(self):
         """Creates a new long lived upload token for use uploading jobs
@@ -466,7 +528,7 @@ class GWCloud:
         """
 
         data = self.request(query=query)
-        return data['generateBilbyJobUploadToken']['token']
+        return data['generate_bilby_job_upload_token']['token']
 
     def upload_job_archive(self, description, job_archive, public=False):
         """Upload a bilby job to GWCloud by job output archive
@@ -511,7 +573,7 @@ class GWCloud:
 
             data = self.request(query=query, variables=variables, authorize=False)
 
-        job_id = data['uploadBilbyJob']['result']['jobId']
+        job_id = data['upload_bilby_job']['result']['job_id']
         return self.get_job_by_id(job_id)
 
     def upload_job_directory(self, description, job_directory, public=False):
@@ -583,7 +645,7 @@ class GWCloud:
             }
         }
         data = self.request(query=query, variables=variables)
-        logger.info(data['createEventId']['result'])
+        logger.info(data['create_event_id']['result'])
         return self.get_event_id(event_id=event_id)
 
     def update_event_id(self, event_id, trigger_id=None, nickname=None, is_ligo_event=None):
@@ -624,7 +686,7 @@ class GWCloud:
             }
         }
         data = self.request(query=query, variables=variables)
-        logger.info(data['updateEventId']['result'])
+        logger.info(data['update_event_id']['result'])
         return self.get_event_id(event_id=event_id)
 
     def delete_event_id(self, event_id):
@@ -651,7 +713,7 @@ class GWCloud:
             }
         }
         data = self.request(query=query, variables=variables)
-        logger.info(data['deleteEventId']['result'])
+        logger.info(data['delete_event_id']['result'])
 
     def get_event_id(self, event_id):
         """Get EventID by the event_id
@@ -683,7 +745,7 @@ class GWCloud:
             "eventId": event_id
         }
         data = self.request(query=query, variables=variables)
-        return EventID(**convert_dict_keys(data['eventId']))
+        return EventID(**data['event_id'])
 
     def get_all_event_ids(self):
         """Obtain a list of all Event IDs
@@ -709,4 +771,4 @@ class GWCloud:
             }
         """
         data = self.request(query=query)
-        return [EventID(**convert_dict_keys(event)) for event in data['allEventIds']]
+        return [EventID(**event) for event in data['all_event_ids']]
